from enum import Enum
from sys import path_hooks
from xmlrpc.client import UNSUPPORTED_ENCODING
from pydbus import SystemBus
from gi.repository import GLib, Gtk, Gio
import json
import ctypes
from ctypes import *
from struct import pack

class classifyret(Enum):
    ERR = -1
    OK = 0
    UNSUPPORTED = 114119


class resourceType(Enum):
    RESOURCE_CORE = 0
    RESOURCE_SYSTEM = 1
    RESOURCE_USER =2
    RESOURCE_APP = 3

class appType(Enum):
    APP_SYSTEM = 0
    APP_APPLICATION = 1

class classifyStatus(Enum):
    STATUS_ON = 0
    STATUS_OFF = 1


class kysecKidType(Enum):
    kid_type_user = 0           #用户
    kid_type_core = 1           #核心
    kid_type_system = 2         #系统
    kid_type_app = 3            #应用软件
    kid_type_tmp = 4            #临时



class resourceStruct(Structure):
    _fields_ = (
        ('kidType', c_int),
        ('type', c_int),
        ('resource', c_char*4096),
    )


class appStruct(Structure):
    _fields_ = (
        ('source', c_char*128),
        ('app', c_char*128),
        ('type', c_int),
    )

class policyStruct(Structure):
    _fields_ = (
        ('subType', c_int),
        ('subject', c_char*4096),
        ('object', c_char*4096),
        ('perm', c_int),
    )

class netctlStruct(Structure):
    _fields_ = (
        ('type', c_int),
        ('subject', c_char*4096),
    )

class classify:
    """
    <node>
        <interface name='com.kylin.kysdk.applicationsec.classify'>
            <method name='getStatus' >
                <arg type='i' direction='out' name='ret'/>
            </method>
            <method name='setStatus' >
                <arg type='i' direction='in' name='status'/>
                <arg type='i' direction='out' name='ret'/>
            </method>


            <method name='getResourceList' >
                <arg type='i' direction='in' name='reType'/>
                <arg type='s' direction='out' name='list'/>
            </method>
            <method name='addResource' >
                <arg type='s' direction='in' name='resource'/>
                <arg type='i' direction='in' name='resourceType'/>
                <arg type='i' direction='out' name='ret'/>
            </method>
            <method name='delResource' >
                <arg type='s' direction='in' name='resource'/>
                <arg type='i' direction='in' name='resourceType'/>
                <arg type='i' direction='out' name='ret'/>
            </method>
            <method name='updateResource' >
                <arg type='s' direction='in' name='resource'/>
                <arg type='i' direction='in' name='resourceType'/>
                <arg type='i' direction='out' name='ret'/>
            </method>



            <method name='getAppList' >
                <arg type='i' direction='in' name='appType'/>
                <arg type='s' direction='out' name='list'/>
            </method>
            <method name='getInstalledAppList' >
                <arg type='s' direction='out' name='list'/>
            </method>
            <method name='addApp' >
                <arg type='s' direction='in' name='app'/>
                <arg type='s' direction='in' name='source'/>
                <arg type='i' direction='out' name='ret'/>
            </method>
            <method name='delApp' >
                <arg type='s' direction='in' name='app'/>
                <arg type='i' direction='out' name='ret'/>
            </method>
            <method name='updateApp' >
                <arg type='s' direction='in' name='app'/>
                <arg type='i' direction='in' name='dstType'/>
                <arg type='i' direction='out' name='ret'/>
            </method>


            <method name='addPolicy' >
                <arg type='i' direction='in' name='subType'/>
                <arg type='s' direction='in' name='subject'/>
                <arg type='s' direction='in' name='object'/>
                <arg type='i' direction='in' name='perm'/>
                <arg type='i' direction='out' name='ret'/>
            </method>
            <method name='delPolicy' >
                <arg type='i' direction='in' name='subType'/>
                <arg type='s' direction='in' name='subject'/>
                <arg type='s' direction='in' name='object'/>
                <arg type='i' direction='out' name='ret'/>
            </method>
            <method name='updatePolicy' >
                <arg type='i' direction='in' name='subType'/>
                <arg type='s' direction='in' name='subject'/>
                <arg type='s' direction='in' name='object'/>
                <arg type='i' direction='in' name='perm'/>
                <arg type='i' direction='out' name='ret'/>
            </method>
            <method name='getPolicyList' >
                <arg type='s' direction='out' name='list'/>
            </method>
            <method name='getPolicyListBySub' >
                <arg type='i' direction='in' name='subType'/>
                <arg type='s' direction='in' name='subject'/>
                <arg type='s' direction='out' name='list'/>
            </method>


            <method name='addNetctl' >
                <arg type='i' direction='in' name='subType'/>
                <arg type='s' direction='in' name='subject'/>
                <arg type='i' direction='out' name='ret'/>
            </method>
            <method name='delNetctl' >
                <arg type='i' direction='in' name='subType'/>
                <arg type='s' direction='in' name='subject'/>
                <arg type='i' direction='out' name='ret'/>
            </method>
            <method name='getNetctlList' >
                <arg type='s' direction='out' name='list'/>
            </method>

            <method name='addKysecConf' >
                <arg type='i' direction='out' name='ret'/>
            </method>
            <method name='setKysecConf' >
                <arg type='s' direction='in' name='key'/>
                <arg type='i' direction='in' name='value'/>
                <arg type='i' direction='out' name='ret'/>
            </method>
            

            <method name='addPrivacyRes' >
                <arg type='s' direction='in' name='resource'/>
                <arg type='i' direction='out' name='ret'/>
            </method>
            <method name='delPrivacyRes' >
                <arg type='s' direction='in' name='resource'/>
                <arg type='i' direction='out' name='ret'/>
            </method>

        </interface>
    </node>
    """
    

    def __init__(self):        
        self.kysec_object = CDLL("libkysecwhlist.so.0.0.0")
        print("+++++++++++init done")

        self.kysec_extend_object = CDLL("libkysec_extend.so.0.0.0")
        print("+++++++++++kysec_extend init done")

        self.reTypeToKysec = {
            resourceType.RESOURCE_CORE.value : kysecKidType.kid_type_core.value, \
            resourceType.RESOURCE_SYSTEM.value : kysecKidType.kid_type_system.value, \
            resourceType.RESOURCE_USER.value : kysecKidType.kid_type_user.value, \
            resourceType.RESOURCE_APP.value : kysecKidType.kid_type_app.value, \
        }

        self.KysecToreType = {
            kysecKidType.kid_type_core.value : resourceType.RESOURCE_CORE.value, \
            kysecKidType.kid_type_system.value : resourceType.RESOURCE_SYSTEM.value, \
            kysecKidType.kid_type_user.value : resourceType.RESOURCE_USER.value , \
            kysecKidType.kid_type_app.value : resourceType.RESOURCE_APP.value, \
        }


        self.appTypeToKysec = {
            appType.APP_SYSTEM.value : kysecKidType.kid_type_system.value,
            appType.APP_APPLICATION.value : kysecKidType.kid_type_app.value,
        }

        self.kysecToAppType = {
            kysecKidType.kid_type_system.value : appType.APP_SYSTEM.value, \
            kysecKidType.kid_type_app.value : appType.APP_APPLICATION.value, \
        }


    '''
    #获取能力集
    def getCapability(self):
        #返回资源分类信息及能力
        pass
    '''
    #获取状态 0 关闭， 1 强制模式（开启权限分级，策略为向量表策略），
    #2  兼容模式（开启权限分级，策略为内核策略和向量表策略）， 其他 失败
    def getStatus(self):
        return self.kysec_object.kysec_kid_whlist_func_status_get()
    
    #设置状态 return int 0 成功， 其他 失败
    def setStatus(self, status):
        return self.kysec_object.kysec_kid_whlist_func_status_set(status)


    '''
    #设置数据库安全性
    def setDbSec():
        #文件防篡改+数据库加密
        pass

    #系统初始化
    def systemInit(self):
        pass
    
    #获取资源分类的基础权限策略
    def getClassifyPermissionPolicy():
        #抽象基础策略。
        pass

    #设置资源分类的基础权限策略
    def setClassifyPermissionPolicy():
        pass
    '''

    '''
            <method name='getAppResourceMap' >
                <arg type='s' direction='out' name='list'/>
            </method>

    #获取应用分类与资源分类映射关系
    def getAppResourceMap(self):
        jsonList = 'UNSUPPORTED'
        return jsonList
        #'[["1", 2], ["0", 3]]'
    '''

    '''
            <method name='setDefaultState' >
                <arg type='i' direction='in' name='state'/>
                <arg type='i' direction='out' name='ret'/>
            </method>
            <method name='setDefaultResourceType' >
                <arg type='i' direction='in' name='type'/>
                <arg type='i' direction='out' name='ret'/>
            </method>
            <method name='setDefaultAppType' >
                <arg type='i' direction='in' name='type'/>
                <arg type='i' direction='out' name='ret'/>
            </method>

    #设置默认状态
    def setDefaultState(self, state):
        return classifyret.UNSUPPORTED
 
    #设置默认资源类型
    def setDefaultResourceType(self, type):
        return classifyret.UNSUPPORTED

    #设置默认应用类型
    def setDefaultAppType(self, type):
        return classifyret.UNSUPPORTED
    '''

    #获取系统资源分类列表
    def getResourceList(self, reType):
        reType = self.reTypeToKysec.get(reType)
        print("+++++++++++ddd", reType)

        count = ctypes.c_int()
        commonStruct = POINTER(resourceStruct)()
        self.kysec_object.kysec_kid_whlist_load_res_list_v2(reType, 0,byref(commonStruct), byref(count))

        allList = []
        for n in range(0, count.value):
            kidType = self.KysecToreType.get(commonStruct[n].kidType)
            tempList = [commonStruct[n].resource.decode('utf-8'), \
            kidType, commonStruct[n].type]

            allList.append(tempList)
        
        jsonList = json.dumps(allList, ensure_ascii=False)
        print("jsonList=", jsonList)
        return jsonList
        

    #设置系统资源分类列表
    #def setResourceList(self, resourceType, resourceList):
        #pass

    #添加资源
    #pathType 0:文件或目录本身 1：只针对目录下的一层子文件或目录 2：目录下所有文件或目录
    def addResource(self, resource, resourceType):
        pathType = 0 #kysec接口，目前不支持1、2
        resource = create_string_buffer(resource.encode())
        return self.kysec_object.kysec_kid_whlist_add_res(
            self.reTypeToKysec.get(resourceType), pathType, resource)

    #删除资源
    def delResource(self, resource, resourceType):
        pathType = 0
        resource = create_string_buffer(resource.encode())
        return self.kysec_object.kysec_kid_whlist_remove_res(
            self.reTypeToKysec.get(resourceType), pathType, resource)

    #修改资源等级
    def updateResource(self, resource, resourceType):
        pathType = 0
        resource = create_string_buffer(resource.encode())
        return self.kysec_object.kysec_kid_whlist_add_res(
            self.reTypeToKysec.get(resourceType), pathType, resource)

    

    '''
    <method name='queryResource' >
    <arg type='s' direction='in' name='resource'/>
    <arg type='s' direction='out' name='list'/>
    </method>

    #查询资源详细信息
    def queryResource(self, resource):
        jsonList = 'UNSUPPORTED'
        return jsonList
        #'[["/dev/cd", 2]]'
    '''


    #获取应用分类列表
    def getAppList(self, appType):
        if appType == 110:
            testList= '[["pksav", "pksav", 1], ["qaxsafe", "qaxsafe", 1], ["tg-support", "tg-support", 1], ["wps-office", "wps-office", 1], ["crossover", "crossover", 1], ["360zip", "360zip", 1], ["accessagent", "accessagent", 1], ["atril", "atril", 1], ["audacious", "audacious", 1], ["audio-recorder", "audio-recorder", 1], ["baidunetdisk", "baidunetdisk", 1], ["browser360-cn-beta", "browser360-cn-beta", 1], ["browser360-cn-stable", "browser360-cn-stable", 1], ["bytedance-feishu-stable", "bytedance-feishu-stable", 1], ["cajviewer", "cajviewer", 1], ["ccwork", "ccwork", 1], ["cheese", "cheese", 1], ["chromium-browser", "chromium-browser", 1], ["cimfax", "cimfax", 1], ["claws-mail", "claws-mail", 1], ["cn.com.10jqka", "cn.com.10jqka", 1], ["cn.com.topsec.edragent", "cn.com.topsec.edragent", 1], ["cn.gov.gdzwfw.yzy", "cn.gov.gdzwfw.yzy", 1], ["cn.huorong.esm", "cn.huorong.esm", 1], ["cn.kdocs.kdesktop", "cn.kdocs.kdesktop", 1], ["cn.lanxin", "cn.lanxin", 1], ["com.360.epp", "com.360.epp", 1], ["com.alibabainc.dingtalk", "com.alibabainc.dingtalk", 1], ["com.fenglinkeji.scanner", "com.fenglinkeji.scanner", 1], ["com.foxit.foxitpdfeditor", "com.foxit.foxitpdfeditor", 1], ["com.kuaicad.cadreader", "com.kuaicad.cadreader", 1], ["com.lenovo.ktservice", "com.lenovo.ktservice", 1], ["com.qq.docs.tdappdesktop", "com.qq.docs.tdappdesktop", 1], ["com.rising.antivirus.single", "com.rising.antivirus.single", 1], ["com.strato.webprinter", "com.strato.webprinter", 1], ["com.wind.wft", "com.wind.wft", 1], ["com.xunlei.download", "com.xunlei.download", 1], ["dianjureader", "dianjureader", 1], ["eclipse", "eclipse", 1], ["edrawmax", "edrawmax", 1], ["engrampa", "engrampa", 1], ["eom", "eom", 1], ["evolution", "evolution", 1], ["eyecool-facesdk", "eyecool-facesdk", 1], ["filezilla", "filezilla", 1], ["firefox", "firefox", 1], ["firefox-esr", "firefox-esr", 1], ["fsmeeting", "fsmeeting", 1], ["gimp", "gimp", 1], ["gnome-paint", "gnome-paint", 1], ["gnome-tetravex", "gnome-tetravex", 1], ["google-chrome-stable", "google-chrome-stable", 1], ["gpicview", "gpicview", 1], ["himirage", "himirage", 1], ["htbrowser", "htbrowser", 1], ["huijian", "huijian", 1], ["huluer", "huluer", 1], ["igspc", "igspc", 1], ["indicator-china-weather", "indicator-china-weather", 1], ["kylin-browser", "kylin-browser", 1], ["kylin-calculator", "kylin-calculator", 1], ["kylin-camera", "kylin-camera", 1], ["kylin-cloud-printer-client", "kylin-cloud-printer-client", 1], ["kylin-ipmsg", "kylin-ipmsg", 1], ["kylin-music", "kylin-music", 1], ["kylin-scanner", "kylin-scanner", 1], ["kylinsemail-client-ky10", "kylinsemail-client-ky10", 1], ["kylin-video", "kylin-video", 1], ["kylin-weather", "kylin-weather", 1], ["lbrowser", "lbrowser", 1], ["libpinpad", "libpinpad", 1], ["libreoffice", "libreoffice", 1], ["linuxqq", "linuxqq", 1], ["mate-calc", "mate-calc", 1], ["mate-media", "mate-media", 1], ["mate-system-monitor", "mate-system-monitor", 1], ["mate-utils", "mate-utils", 1], ["microsoft-edge-beta", "microsoft-edge-beta", 1], ["microsoft-edge-stable", "microsoft-edge-stable", 1], ["mindmaster", "mindmaster", 1], ["mitalk", "mitalk", 1], ["mpv", "mpv", 1], ["mtxx", "mtxx", 1], ["netease-cloud-music", "netease-cloud-music", 1], ["pluma", "pluma", 1], ["qaxbrowser-pioneer-stable", "qaxbrowser-pioneer-stable", 1], ["qaxbrowser-safe-stable", "qaxbrowser-safe-stable", 1], ["qqbrowser", "qqbrowser", 1], ["recoll", "recoll", 1], ["rededa", "rededa", 1], ["sc-reader", "sc-reader", 1], ["simple-scan", "simple-scan", 1], ["skypeforlinux", "skypeforlinux", 1], ["smplayer", "smplayer", 1], ["suwellreaderpro.appstore", "suwellreaderpro.appstore", 1], ["thunderbird", "thunderbird", 1], ["transmission", "transmission", 1], ["ts-suite", "ts-suite", 1], ["udssmartscan", "udssmartscan", 1], ["uget", "uget", 1], ["viewpdf.app", "viewpdf.app", 1], ["vrvsd", "vrvsd", 1], ["wechat", "wechat", 1], ["weixin", "weixin", 1], ["wemeet", "wemeet", 1], ["wwlocal", "wwlocal", 1], ["xmind-vana", "xmind-vana", 1], ["xsvr-hibox", "xsvr-hibox", 1], ["ynote-desktop", "ynote-desktop", 1], ["youku-app", "youku-app", 1], ["yozo-office", "yozo-office", 1], ["zwcad2023", "zwcad2023", 1], ["dangtest", "dang", 1]]'
            return testList
            
        count = ctypes.c_int()
        commonStruct = POINTER(appStruct)()
        self.kysec_object.kysec_kid_whlist_load_app_list_v2(byref(commonStruct), byref(count))

        allList = []
        for n in range(0, count.value):
            type = self.kysecToAppType.get(commonStruct[n].type)
            if type == appType:
                tempList = [commonStruct[n].app.decode('utf-8'),
                    commonStruct[n].source.decode('utf-8'), type]
            
                allList.append(tempList)
        
        jsonList = json.dumps(allList, ensure_ascii=False)
        print("jsonList=", jsonList)
        return jsonList


        '''
        list1 = ['kylin-log-viewer', 'kylin-log-viewer', 1]
        list2 = ['ksc-defender', 'ksc-defender', 0]
        list3 = ['wechat', 'wechat', 1]
        applist = []
        applist.append(list1)
        applist.append(list2)
        applist.append(list3)

        print("appList=",applist)

        jsonList = json.dumps(applist, ensure_ascii=False)
        print("jsonList=", jsonList)

        return jsonList
        '''
        #'[["logview", 1], ["box", 0], ["defender", 0], ["weather", 1], ["notify", 1], '
        #'["ddf", 1], ["qianxin", 1], ["liulanqi", 1]]'

    #获取已安装应用分类列表
    def getInstalledAppList(self):
        appType = 1
        count = ctypes.c_int()
        commonStruct = POINTER(appStruct)()
        self.kysec_object.kysec_kid_whlist_load_installed_app_list_v2(byref(commonStruct), byref(count))

        allList = []
        for n in range(0, count.value):
            type = self.kysecToAppType.get(commonStruct[n].type)
            if type == appType:
                tempList = [commonStruct[n].app.decode('utf-8'),
                    commonStruct[n].source.decode('utf-8'), type]
            
                allList.append(tempList)
        
        jsonList = json.dumps(allList, ensure_ascii=False)
        print("jsonList=", jsonList)
        return jsonList




    #增加应用
    #app:应用名   source:源码包名
    def addApp(self, app, source):
        app = create_string_buffer(app.encode())
        source = create_string_buffer(source.encode())
        #self.kysec_object.kysec_kid_whlist_create_kid(source, app, POINTER(c_char)(), 0)
        return self.kysec_object.kysec_kid_whlist_create_kid(source, app, c_char_p(), 0)

    #删除应用
    def delApp(self, app):
        app = create_string_buffer(app.encode())
        return self.kysec_object.kysec_kid_whlist_remove_kid(app)


    #修改应用级别
    def updateApp(self, app, dstType):
        dstType = self.appTypeToKysec.get(dstType)
        app = create_string_buffer(app.encode())
        return self.kysec_object.kysec_kid_whlist_improve_kid(dstType, app)


    '''
            <method name='queryApp' >
                <arg type='s' direction='in' name='app'/>
                <arg type='s' direction='out' name='list'/>
            </method>

    #查询应用分类信息
    def queryApp(self, app):
        jsonList = 'UNSUPPORTED'
        return jsonList
    
    '''


    # 添加策略
    # subType 主体类型    0:应用  1：文件、程序
    # subject策略主体
    # object 策略客体 perm权限 0x1111重命名、删除、写、读
    def addPolicy(self, subType, subject, object, perm):
        subject = create_string_buffer(subject.encode())
        object = create_string_buffer(object.encode())
        return self.kysec_object.kysec_kid_whlist_add_kid_policy(subType, subject, object, perm)

    #移除策略
    def delPolicy(self, subType, subject, object):
        subject = create_string_buffer(subject.encode())
        object = create_string_buffer(object.encode())
        return self.kysec_object.kysec_kid_whlist_remove_kid_policy(subType, subject, object)

    #更新策略
    def updatePolicy(self, subType, subject, object, perm):
        subject = create_string_buffer(subject.encode())
        object = create_string_buffer(object.encode())
        return self.kysec_object.kysec_kid_whlist_update_kid_policy(subType, subject, object, perm)

    #获取策略
    def getPolicyList(self):
        count = ctypes.c_int()
        commonStruct = POINTER(policyStruct)()
        self.kysec_object.kysec_kid_whlist_load_policy_list_v2(byref(commonStruct), byref(count))

        allList = []
        for n in range(0, count.value):
            tempList = [commonStruct[n].subType, commonStruct[n].subject.decode('utf-8'), \
            commonStruct[n].object.decode('utf-8'), commonStruct[n].perm]
            allList.append(tempList)
        
        jsonList = json.dumps(allList, ensure_ascii=False)
        print("jsonList=", jsonList)
        return jsonList

    #根据主体获取策略
    def getPolicyListBySub(self, subType, subject):
        count = ctypes.c_int()
        subject = create_string_buffer(subject.encode())
        commonStruct = POINTER(policyStruct)()
        ret = self.kysec_object.kysec_kid_whlist_load_policy_list_by_subject_v2(subType, subject, byref(commonStruct), byref(count))

        allList = []
        for n in range(0, count.value):
            tempList = [commonStruct[n].subType, commonStruct[n].subject.decode('utf-8'), \
            commonStruct[n].object.decode('utf-8'), commonStruct[n].perm]
            allList.append(tempList)
        
        jsonList = json.dumps(allList, ensure_ascii=False)
        print("jsonList=", jsonList)
        return jsonList



    #增加联网策略
    def addNetctl(self, subType, subject):
        subject = create_string_buffer(subject.encode())
        return self.kysec_object.kysec_kid_whlist_add_kid_netctl(subType, subject)

    #删除联网策略
    def delNetctl(self, subType, subject):
        subject = create_string_buffer(subject.encode())
        return self.kysec_object.kysec_kid_whlist_remove_kid_netctl(subType, subject)

    #获取联网列表
    def getNetctlList(self):
        count = ctypes.c_int()
        commonStruct = POINTER(netctlStruct)()
        self.kysec_object.kysec_kid_whlist_load_netctl_list_v2(byref(commonStruct), byref(count))

        allList = []
        for n in range(0, count.value):
            tempList = [commonStruct[n].subject.decode('utf-8'), commonStruct[n].type]
            allList.append(tempList)
        
        jsonList = json.dumps(allList, ensure_ascii=False)
        print("jsonList=", jsonList)
        return jsonList

    #添加kysec配置
    def addKysecConf(self):
        return self.kysec_extend_object.kysec_conf_add()

    #添加kysec配置
    def setKysecConf(self, key, value):
        key = create_string_buffer(key.encode())
        return self.kysec_extend_object.kysec_conf_set(key, value)


    #添加用户隐私资源
    def addPrivacyRes(self, resource):
        resource = create_string_buffer(resource.encode())
        return self.kysec_object.kysec_kid_whlist_add_user_privacy_res(resource)

    #刪除用户隐私资源
    def delPrivacyRes(self, resource):
        resource = create_string_buffer(resource.encode())
        return self.kysec_object.kysec_kid_whlist_remove_user_privcay_res(resource)



if __name__ == "__main__":
    '''
    cl = classify()
    cl.updateApp("/eee", 4)
    
    '''
    loop = GLib.MainLoop()
    bus = SystemBus()
    bus.publish("com.kylin.kysdk.applicationsec", classify())
    loop.run()
    # '''